char const *sys_siglist[] = {
  "no signal",
  "Hangup",
  "Interrupt",
  "Quit",
  "Illegal Instruction",
  "Trace Trap",
  "Abort",
  "Privilege Violation",
  "Floating Point Exception",
  "Killed",
  "Bus Error",
  "Segmentation Fault",
  "Bad Argument to System Call",
  "Broken Pipe",
  "Alarm Clock",
  "Terminated",

  "Urgent Condition on I/O Channel",
  "Stopped (signal)",
  "Stopped",
  "Continued",
  "Child Process Exited",
  "Stopped (tty input)",
  "Stopped (tty output)",
  "I/O Possible",
  "CPU Limit Exceeded",
  "File Size Limit Exceeded",
  "Virtual Timer Alarm",
  "Profiling Signal",
  "Window System Signal",
  "User-defined Signal 1",
  "User-defined Signal 2"
};

char const *signal_names[] = {
  "ON_EXIT",
  "SIGHUP",
  "SIGINT",
  "SIGQUIT",
  "SIGILL",
  "SIGTRAP",
  "SIGABRT",
  "SIGPRIV",
  "SIGFPE",
  "SIGKILL",
  "SIGBUS",
  "SIGSEGV",
  "SIGSYS",
  "SIGPIPE",
  "SIGALRM",
  "SIGTERM",

  "SIGURG",
  "SIGSTOP",
  "SIGTSTP",
  "SIGCONT",
  "SIGCHLD",
  "SIGTTIN",
  "SIGTTOU",
  "SIGIO",
  "SIGXCPU",
  "SIGXFSZ",
  "SIGVTALRM",
  "SIGPROF",
  "SIGWINCH",
  "SIGUSR1",
  "SIGUSR2"
};
