#include <stddef.h>
#include <stdlib.h>
#include <assert.h>

__EXTERN long strtol __PROTO((const char *, char **, int));

int
atoi(str)
const char *str;
{
	assert ((str != NULL));
	return (int) strtol(str, (char **)0, 10);
}
