	INCLUDE	inc.i
	
; Line A bindings for Lattice C

	INFO
	xref	___aline,___fonts,___funcs
	
	CODE
	IF	_REGARG
	xdef	@linea0
@linea0:
	ELSE
	xdef	_linea0
_linea0:
	ENDC
	
	movem.l	d2/a2,-(sp)
	dc.w	$A000
	move.l	a0,___aline
	move.l	a1,___fonts
	move.l	a2,___funcs
	movem.l	(sp)+,d2/a2
	rts
	end
