; 5/2/92 sb
; Long-allocation routines.  As per Eric Smith's advice, they are actually
; implemented as _malloc(), _realloc() and _calloc() over in malloc.c in
; order to reduce name-space pollution; here we're just giving them
; "real" names.

	.text
	.globl	_lmalloc
; void *lmalloc(unsigned long n)
_lmalloc:
	jmp	__malloc

	.globl	_lrealloc
; void *lrealloc(void *r, unsigned long n)
_lrealloc:
	jmp	__realloc

	.globl	_lcalloc
; void *calloc(unsigned long n, unsigned long size)
_lcalloc:
	jmp	__calloc
