; 12/22/92 sb -- HSC attempts to use .comm to reserve the space for
; __initial_stack; unfortunately, this flags the symbol as external and
; the loader won't pull it from the library when it needs to resolve the
; symbol.

	.bss

	.globl __initial_stack
__initial_stack:
	.ds.l	1
